--*********************************************************************************************************************
-- This module is used to read the manufacturer IDs from the Microchip 25aa1024 eeprom and the Microchip TC-77
-- Temp sensor

--********************************************************************************************		
library ieee;

use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
entity SPI_CTRL is port(

-- top level pin instantiations.  Note that in this design, uppercase denotes external pins, and lowercase
-- denotes local nets.
--
-- Inputs/ Outputs:



		CLK_6_25:			IN			STD_LOGIC;  --6.25MHz Clock MHz clock
		RESET:				IN			STD_LOGIC;  
      
		TC77_CS:				OUT		STD_LOGIC;	-- 
		TC77_TRI:         OUT   	STD_LOGIC;	-- 
		TC77_SCLK:			OUT		STD_LOGIC;	-- 
		TC77_MISO:			IN			STD_LOGIC;	-- 
		TC77_MOSI:			OUT		STD_LOGIC	

----		AA_1_CS:			OUT		STD_LOGIC;	 
----		AA_1_SCLK:		OUT		STD_LOGIC;	 
----		AA_1_MISO:		IN			STD_LOGIC;	 
----		AA_1_MOSI:		OUT		STD_LOGIC;	 
--		
----		AA_2_CS:			OUT		STD_LOGIC;	 
----		AA_2_SCLK:		OUT		STD_LOGIC;
----		AA_2_MISO:		IN			STD_LOGIC;
----		AA_2_MOSI:		OUT		STD_LOGIC;
--		
----		AA_3_CS:			OUT		STD_LOGIC;
----		AA_3_SCLK:		OUT		STD_LOGIC;
----		AA_3_MISO:		IN			STD_LOGIC;
----		AA_3_MOSI:		OUT		STD_LOGIC; 
		

									);
		
end SPI_CTRL;

architecture rtl of SPI_CTRL is


  

  
  SIGNAL	TC77_CNT		      : STD_LOGIC_VECTOR(3 DOWNTO 0); 
  SIGNAL	TC77_RD_DATA		: STD_LOGIC_VECTOR(15 DOWNTO 0); 
  SIGNAL	TC77_WR_DATA		: STD_LOGIC_VECTOR(15 DOWNTO 0); 
 
  SIGNAL AA_1_CNT		: STD_LOGIC_VECTOR(3 DOWNTO 0);  
  SIGNAL	AA_2_CNT		: STD_LOGIC_VECTOR(3 DOWNTO 0);  
  SIGNAL AA_3_CNT		: STD_LOGIC_VECTOR(3 DOWNTO 0);  
 
								
								
	TYPE 	TC77_mach 		IS (TC77_START, TC77_DROP_CS, FIRST_READ, FIRST_WRITE, SECOND_READ, SECOND_WRITE, TC77_DELAY
								);
	
	
	SIGNAL	TC77_machine		: TC77_mach;	










		
begin 

	


TC77_SCLK <= not CLK_6_25;


	PROCESS(CLK_6_25, RESET)
	BEGIN
		IF RESET = '1' THEN
			TC77_CS   		<= '1';	-- 
		   TC77_TRI  		<= '0';
		   TC77_MOSI 		<= '0';
		   TC77_CNT       <= "0000";   -- Load 16 into cnt	
			TC77_RD_DATA   <= x"0000";
			TC77_WR_DATA   <= x"ffff";
			TC77_machine	<= TC77_START;

		ELSIF CLK_6_25'event AND CLK_6_25 = '1' THEN			
			CASE TC77_machine is 
				WHEN TC77_START  =>  				 									
						TC77_CS   		<= '1';	-- 
						TC77_TRI  		<= '0';
						TC77_MOSI 		<= '0';	
	               TC77_CNT       <= "0000";   -- Load 0000 into cnt
						TC77_RD_DATA   <= x"0000";
						TC77_WR_DATA   <= x"ffff";
						TC77_machine   <= TC77_DROP_CS;

				WHEN TC77_DROP_CS =>  			--wait a few clocks after reset to ensure the fifos are ready.
						TC77_CS   		<= '0';	-- 
						TC77_TRI  		<= '0';
						TC77_MOSI 		<= '0';	
	               TC77_CNT       <= "0000";   -- Load 16 into cnt
						TC77_RD_DATA   <= x"0000";
						TC77_WR_DATA   <= x"ffff";
						TC77_machine   <= FIRST_READ;
					
				WHEN FIRST_READ =>  
	              if TC77_CNT = "1111" then			
						TC77_CS   		<= '0';	-- 
						TC77_TRI  		<= '0';
						TC77_MOSI 		<= '0';	
	               TC77_CNT       <= "0000";   -- Load 16 into cnt
						TC77_RD_DATA   <= TC77_RD_DATA;
						TC77_WR_DATA   <= x"ffff";
						TC77_machine   <= FIRST_WRITE;
                 else
					   TC77_CS   		<= '0';	-- 
						TC77_TRI  		<= '0';
						TC77_MOSI 		<= '0';	
	               TC77_CNT       <= TC77_CNT + 1;   -- Load 16 into cnt
						TC77_RD_DATA   <= TC77_RD_DATA(14 downto 0) & TC77_MISO;
						TC77_WR_DATA   <= x"ffff";
						TC77_machine   <= FIRST_READ;
                  end if;
						  
				 WHEN FIRST_WRITE =>  
	              if TC77_CNT = "1111" then			
						TC77_CS   		<= '0';	-- 
						TC77_TRI  		<= '0';
						TC77_MOSI 		<= '0';	
	               TC77_CNT       <= "0000";   -- Load 16 into cnt
						TC77_RD_DATA   <= TC77_RD_DATA;
						TC77_WR_DATA   <= x"ffff";
						TC77_machine   <= SECOND_READ;
                 else
					   TC77_CS   		<= '0';	-- 
						TC77_TRI  		<= '1';
						TC77_MOSI 		<= TC77_WR_DATA(15);	
	               TC77_CNT       <= TC77_CNT + 1;   -- Load 16 into cnt
						TC77_RD_DATA   <= TC77_RD_DATA;
						TC77_WR_DATA   <= TC77_WR_DATA(14 downto 0) & '0';
						TC77_machine   <= FIRST_WRITE;
                  end if;
					 
				 WHEN SECOND_READ =>  
	              if TC77_CNT = "1111" then			
						TC77_CS   		<= '0';	-- 
						TC77_TRI  		<= '0';
						TC77_MOSI 		<= '0';	
	               TC77_CNT       <= "0000";   -- Load 16 into cnt
						TC77_RD_DATA   <= TC77_RD_DATA;
						TC77_WR_DATA   <= x"0000";
						TC77_machine   <= SECOND_WRITE;
                 else
					   TC77_CS   		<= '0';	-- 
						TC77_TRI  		<= '0';
						TC77_MOSI 		<= '0';	
	               TC77_CNT       <= TC77_CNT + 1;   -- Load 16 into cnt
						TC77_RD_DATA   <= TC77_RD_DATA(14 downto 0) & TC77_MISO;
						TC77_WR_DATA   <= x"ffff";
						TC77_machine   <= SECOND_READ;		
					   end if;
					
			   WHEN SECOND_WRITE =>  
	              if TC77_CNT = "1111" then			
						TC77_CS   		<= '0';	-- 
						TC77_TRI  		<= '0';
						TC77_MOSI 		<= '0';	
	               TC77_CNT       <= "0000";   -- Load 16 into cnt
						TC77_RD_DATA   <= TC77_RD_DATA;
						TC77_WR_DATA   <= x"ffff";
						TC77_machine   <= TC77_DELAY;
                 else
					   TC77_CS   		<= '0';	-- 
						TC77_TRI  		<= '1';
						TC77_MOSI 		<= TC77_WR_DATA(15);	
	               TC77_CNT       <= TC77_CNT + 1;   -- Load 16 into cnt
						TC77_RD_DATA   <= TC77_RD_DATA;
						TC77_WR_DATA   <= TC77_WR_DATA(14 downto 0) & '0';
						TC77_machine   <= SECOND_WRITE; 	
				      end if;
				
				WHEN TC77_DELAY =>  			
						if TC77_CNT = "1111" then			
						TC77_CS   		<= '1';	-- 
						TC77_TRI  		<= '0';
						TC77_MOSI 		<= '0';	
	               TC77_CNT       <= "0000";   -- Load 16 into cnt
						TC77_RD_DATA   <= TC77_RD_DATA;
						TC77_WR_DATA   <= TC77_WR_DATA;
						TC77_machine   <= TC77_START;
                 else
					   TC77_CS   		<= '1';	-- 
						TC77_TRI  		<= '0';
						TC77_MOSI 		<= '0';	
	               TC77_CNT       <= TC77_CNT + 1;   -- Load 16 into cnt
						TC77_RD_DATA   <= TC77_RD_DATA;
						TC77_WR_DATA   <= TC77_WR_DATA;
						TC77_machine   <= TC77_DELAY; 	
				      end if;

--****************************************************************************************************

				WHEN OTHERS =>					
						TC77_CS   		<= '1';	-- 
						TC77_TRI  		<= '0';
						TC77_MOSI 		<= '0';
						TC77_CNT       <= "0000";   -- Load 16 into cnt	
						TC77_RD_DATA   <= x"0000";
						TC77_WR_DATA   <= x"ffff";
						TC77_machine	<= TC77_START;

			END CASE;
		END IF;
	END PROCESS;






END rtl;	