--*********************************************************************************************************************
-- This module is used to read the manufacturer IDs from the Microchip 25aa1024 eeprom and the Microchip TC-77
-- Temp sensor

--********************************************************************************************		
library ieee;

use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
entity SPI_CTRL_25AA is port(

-- top level pin instantiations.  Note that in this design, uppercase denotes external pins, and lowercase
-- denotes local nets.
--
-- Inputs/ Outputs:



		CLK_6_25:			IN			STD_LOGIC;  --6.25MHz Clock MHz clock
		RESET:				IN			STD_LOGIC;  
      
		
		AA_1_CS:			OUT		STD_LOGIC;	 
		AA_1_SCLK:		    OUT		STD_LOGIC;	 
		AA_1_MISO:		    IN			STD_LOGIC;	 
		AA_1_MOSI:		    OUT		STD_LOGIC	 
		
		
		

									);
		
end SPI_CTRL_25AA;

architecture rtla of SPI_CTRL_25AA is


  

  
 
  SIGNAL    EEP_1_CNT		    : STD_LOGIC_VECTOR(4 DOWNTO 0);   
  SIGNAL	EEP_1_RD_DATA		: STD_LOGIC_VECTOR(7 DOWNTO 0); 
  SIGNAL	EEP_1_WR_DATA		: STD_LOGIC_VECTOR(7 DOWNTO 0); 
  
  
								
								
	


	TYPE 	EEPROM_mach 		IS (EEP_START, EEP_DROP_CS, EEP_FIRST_WRITE, EEP_ADD_WRITE, EEP_FIRST_READ,  EEP_DELAY
								);
	
	
	
	SIGNAL	EEPROM_machine		: EEPROM_mach;	


 
		
begin 

	

------------------------------------------------------------------------------------------------------------------
--                                            25AA EEPROM ID Machine
------------------------------------------------------------------------------------------------------------------     

AA_1_SCLK <= not CLK_6_25;


	PROCESS(CLK_6_25, RESET)
	BEGIN
		IF RESET = '1' THEN
			AA_1_CS   		<= '1';	-- 
		    AA_1_MOSI 		<= '0';
		    EEP_1_CNT       <= "00000";  
			EEP_1_RD_DATA   <= x"00";
			EEP_1_WR_DATA   <= x"ff";
			EEPROM_machine	<= EEP_START;

		ELSIF CLK_6_25'event AND CLK_6_25 = '1' THEN			
			CASE EEPROM_machine is 
				WHEN EEP_START  =>  				 									
						AA_1_CS   		<= '1';	-- 
						AA_1_MOSI 		<= '0';	
	               EEP_1_CNT       <= "00000";  
						EEP_1_RD_DATA   <= x"00";
						EEP_1_WR_DATA   <= x"AB";   -- load RDID
						EEPROM_machine   <= EEP_DROP_CS;

				WHEN EEP_DROP_CS =>  			
						AA_1_CS   		<= '0';	-- 
						AA_1_MOSI 		<= EEP_1_WR_DATA(7);	
						EEP_1_CNT       <= "00000";   
						EEP_1_RD_DATA   <= x"00";
						EEP_1_WR_DATA   <= EEP_1_WR_DATA(6 downto 0) & '0';
						EEPROM_machine   <= EEP_FIRST_WRITE;
					
				
						  
				 WHEN EEP_FIRST_WRITE =>  
	              if EEP_1_CNT = "00111" then			
						AA_1_CS   		<= '0';	-- 
						AA_1_MOSI 		<= '0';	
						EEP_1_CNT       <= "00000";   -- clear counter
						EEP_1_RD_DATA   <= EEP_1_RD_DATA;
						EEP_1_WR_DATA   <= x"ff";
						EEPROM_machine   <= EEP_ADD_WRITE;
                 else
					   AA_1_CS   		<= '0';	-- 
						AA_1_MOSI 		<= EEP_1_WR_DATA(7);	
						EEP_1_CNT       <= EEP_1_CNT + 1;   -- increment counter
						EEP_1_RD_DATA   <= EEP_1_RD_DATA;
						EEP_1_WR_DATA   <= EEP_1_WR_DATA(6 downto 0) & '0';
						EEPROM_machine   <= EEP_FIRST_WRITE;
                  end if;
					 
				 
					
			   WHEN EEP_ADD_WRITE =>  
	              if EEP_1_CNT = "10111" then	-- need to write 24 clocks, so 0 thru 23		
						AA_1_CS   		<= '0';	-- 
						AA_1_MOSI 		<= '0';	
						EEP_1_CNT       <= "00000";   
						EEP_1_RD_DATA   <= EEP_1_RD_DATA;
						EEP_1_WR_DATA   <= x"ff";
						EEPROM_machine   <= EEP_FIRST_READ;
                 else
					    AA_1_CS   		<= '0';	-- 
						AA_1_MOSI 		<= '0';  -- Write 0 because its a dont care	
						EEP_1_CNT       <= EEP_1_CNT + 1;   
						EEP_1_RD_DATA   <= EEP_1_RD_DATA;
						EEP_1_WR_DATA   <= x"ff";
						EEPROM_machine   <= EEP_ADD_WRITE; 	
				      end if;
				
				WHEN EEP_FIRST_READ =>  
	              if EEP_1_CNT = "00111" then			
						AA_1_CS   		<= '0';	-- 
						AA_1_MOSI 		<= '0';	
						EEP_1_CNT       <= "00000";   
						EEP_1_RD_DATA   <= EEP_1_RD_DATA;
						EEP_1_WR_DATA   <= x"00";
						EEPROM_machine   <= EEP_DELAY;
                 else
					    AA_1_CS   		<= '0';	-- 
						AA_1_MOSI 		<= '0';	
						EEP_1_CNT       <= EEP_1_CNT + 1;   
						EEP_1_RD_DATA   <= EEP_1_RD_DATA(6 downto 0) & AA_1_MISO;
						EEP_1_WR_DATA   <= x"ff";
						EEPROM_machine   <= EEP_FIRST_READ;		
					   end if;
				
				WHEN EEP_DELAY =>  			
						if EEP_1_CNT = "11111" then			
							AA_1_CS   		<= '1';	-- 
							AA_1_MOSI 		<= '0';	
							EEP_1_CNT       <= "00000";   
							EEP_1_RD_DATA   <= EEP_1_RD_DATA;
							EEP_1_WR_DATA   <= EEP_1_WR_DATA;
							EEPROM_machine   <= EEP_START;
						else
							AA_1_CS   		<= '1';	-- 
							AA_1_MOSI 		<= '0';	
							EEP_1_CNT       <= EEP_1_CNT + 1;   
							EEP_1_RD_DATA   <= EEP_1_RD_DATA;
							EEP_1_WR_DATA   <= EEP_1_WR_DATA;
							EEPROM_machine   <= EEP_DELAY; 	
				      end if;

--****************************************************************************************************

				WHEN OTHERS =>					
						AA_1_CS   		<= '1';	-- 
						AA_1_MOSI 		<= '0';
						EEP_1_CNT       <= "00000";  
						EEP_1_RD_DATA   <= x"00";
						EEP_1_WR_DATA   <= x"ff";
						EEPROM_machine	<= EEP_START;

			END CASE;
		END IF;
	END PROCESS;
-----------------------------------------------------------------------------------------------------------------





END rtla;	